@@ -53,7 +53,7 @@ static PyObject *sqlalchemy_engine_row = NULL;
 static PyObject *sqlalchemy_engine_result = NULL;
 
 
-//static int KEY_INTEGER_ONLY = 0;
+static int KEY_INTEGER_ONLY = 0;
 static int KEY_OBJECTS_ONLY = 1;
 static int KEY_OBJECTS_BUT_WARN = 2;
 //static int KEY_OBJECTS_NO_WARN = 3;
@@ -415,6 +415,7 @@ BaseRow_subscript_impl(BaseRow *self, PyObject *key, int asmapping)
     PyObject *values;
     PyObject *result;
     long index;
+    PyObject *tmp;
 
 #if PY_MAJOR_VERSION < 3
     if (PyInt_CheckExact(key)) {
@@ -466,6 +467,14 @@ BaseRow_subscript_impl(BaseRow *self, PyObject *key, int asmapping)
         result = BaseRow_valuescollection(values, 1);
         Py_DECREF(values);
         return result;
+    }
+    else if (!asmapping && self->key_style == KEY_INTEGER_ONLY) {
+        tmp = PyObject_CallMethod(self->parent, "_raise_for_nonint", "(O)", key);
+        if (tmp == NULL) {
+            return NULL;
+        }
+        Py_DECREF(tmp);
+        return NULL;
     } else {
         return BaseRow_getitem_by_object(self, key, asmapping);
     }
@@ -480,7 +489,7 @@ BaseRow_subscript(BaseRow *self, PyObject *key)
 static PyObject *
 BaseRow_subscript_mapping(BaseRow *self, PyObject *key)
 {
-    if (self->key_style == KEY_OBJECTS_BUT_WARN) {
+    if (self->key_style == KEY_OBJECTS_BUT_WARN || self->key_style == KEY_INTEGER_ONLY) {
         return BaseRow_subscript_impl(self, key, 0);
     }
     else {
