@@ -68,6 +68,7 @@ class DefaultDialect(interfaces.Dialect):
     # not cx_oracle.
     execute_sequence_format = tuple
 
+    supports_schemas = True
     supports_views = True
     supports_sequences = False
     sequences_optional = False
@@ -790,6 +791,7 @@ class StrCompileDialect(DefaultDialect):
 
     supports_native_boolean = True
 
+    supports_multivalues_insert = True
     supports_simple_order_by_label = True
 
     colspecs = {
