@@ -1,5 +1,6 @@
 # sqlalchemy/log.py
-# Copyright (C) 2006-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2006-2021 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 # Includes alterations by Vinay Sajip vinay_sajip@yahoo.co.uk
 #
 # This module is part of SQLAlchemy and is released under
@@ -11,8 +12,8 @@ Control of logging for SA can be performed from the regular python logging
 module.  The regular dotted module namespace is used, starting at
 'sqlalchemy'.  For class-level logging, the class name is appended.
 
-The "echo" keyword parameter, available on SQLA :class:`.Engine`
-and :class:`.Pool` objects, corresponds to a logger specific to that
+The "echo" keyword parameter, available on SQLA :class:`_engine.Engine`
+and :class:`_pool.Pool` objects, corresponds to a logger specific to that
 instance only.
 
 """
@@ -20,32 +21,42 @@ instance only.
 import logging
 import sys
 
+
 # set initial level to WARN.  This so that
 # log statements don't occur in the absence of explicit
 # logging being enabled for 'sqlalchemy'.
-rootlogger = logging.getLogger('sqlalchemy')
+rootlogger = logging.getLogger("sqlalchemy")
 if rootlogger.level == logging.NOTSET:
     rootlogger.setLevel(logging.WARN)
 
 
 def _add_default_handler(logger):
     handler = logging.StreamHandler(sys.stdout)
-    handler.setFormatter(logging.Formatter(
-        '%(asctime)s %(levelname)s %(name)s %(message)s'))
+    handler.setFormatter(
+        logging.Formatter("%(asctime)s %(levelname)s %(name)s %(message)s")
+    )
     logger.addHandler(handler)
 
 
 _logged_classes = set()
 
 
+def _qual_logger_name_for_cls(cls):
+    return (
+        getattr(cls, "_sqla_logger_namespace", None)
+        or cls.__module__ + "." + cls.__name__
+    )
+
+
 def class_logger(cls):
-    logger = logging.getLogger(cls.__module__ + "." + cls.__name__)
+    logger = logging.getLogger(_qual_logger_name_for_cls(cls))
     cls._should_log_debug = lambda self: logger.isEnabledFor(logging.DEBUG)
     cls._should_log_info = lambda self: logger.isEnabledFor(logging.INFO)
     cls.logger = logger
     _logged_classes.add(cls)
     return cls
 
+
 class Identified(object):
     logging_name = None
 
@@ -70,8 +81,8 @@ class InstanceLogger(object):
 
     'debug' -> logging.DEBUG
     True    -> logging.INFO
-    False   -> Effective level of underlying logger
-               (logging.WARNING by default)
+    False   -> Effective level of underlying logger (
+    logging.WARNING by default)
     None    -> same as False
     """
 
@@ -80,7 +91,7 @@ class InstanceLogger(object):
         None: logging.NOTSET,
         False: logging.NOTSET,
         True: logging.INFO,
-        'debug': logging.DEBUG,
+        "debug": logging.DEBUG,
     }
 
     def __init__(self, echo, name):
@@ -89,8 +100,7 @@ class InstanceLogger(object):
 
         # if echo flag is enabled and no handlers,
         # add a handler to the list
-        if self._echo_map[echo] <= logging.INFO \
-            and not self.logger.handlers:
+        if self._echo_map[echo] <= logging.INFO and not self.logger.handlers:
             _add_default_handler(self.logger)
 
     #
@@ -172,11 +182,12 @@ def instance_logger(instance, echoflag=None):
     """create a logger for an instance that implements :class:`.Identified`."""
 
     if instance.logging_name:
-        name = "%s.%s.%s" % (instance.__class__.__module__,
-                      instance.__class__.__name__, instance.logging_name)
+        name = "%s.%s" % (
+            _qual_logger_name_for_cls(instance.__class__),
+            instance.logging_name,
+        )
     else:
-        name = "%s.%s" % (instance.__class__.__module__,
-                  instance.__class__.__name__)
+        name = _qual_logger_name_for_cls(instance.__class__)
 
     instance._echo = echoflag
 
