@@ -4,14 +4,15 @@ import sqlalchemy as sa
 from sqlalchemy import exc as sa_exc
 from sqlalchemy import FetchedValue
 from sqlalchemy import ForeignKey
+from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import String
 from sqlalchemy import testing
 from sqlalchemy.orm import attributes
-from sqlalchemy.orm import create_session
 from sqlalchemy.orm import defer
 from sqlalchemy.orm import deferred
 from sqlalchemy.orm import exc as orm_exc
+from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import lazyload
 from sqlalchemy.orm import make_transient_to_detached
 from sqlalchemy.orm import mapper
@@ -24,6 +25,7 @@ from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing.fixtures import fixture_session
 from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from sqlalchemy.testing.util import gc_collect
@@ -46,7 +48,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(7)
         assert len(u.addresses) == 1
         u.name = "foo"
@@ -64,7 +66,7 @@ class ExpireTest(_fixtures.FixtureTest):
         u.name = "foo"
         sess.flush()
         # change the value in the DB
-        users.update(users.c.id == 7, values=dict(name="jack")).execute()
+        sess.execute(users.update(users.c.id == 7, values=dict(name="jack")))
         sess.expire(u)
         # object isn't refreshed yet, using dict to bypass trigger
         assert u.__dict__.get("name") != "jack"
@@ -80,11 +82,29 @@ class ExpireTest(_fixtures.FixtureTest):
 
         self.assert_sql_count(testing.db, go, 0)
 
+    def test_expire_autoflush(self):
+        User, users = self.classes.User, self.tables.users
+        Address, addresses = self.classes.Address, self.tables.addresses
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={"user": relationship(User)})
+
+        s = fixture_session()
+
+        a1 = s.query(Address).get(2)
+        u1 = s.query(User).get(7)
+        a1.user = u1
+
+        s.expire(a1, ["user_id"])
+
+        # autoflushes
+        eq_(a1.user_id, 7)
+
     def test_persistence_check(self):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(7)
         s.expunge_all()
 
@@ -99,7 +119,7 @@ class ExpireTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session(autocommit=False)
+        s = fixture_session(autocommit=False)
         u = s.query(User).get(10)
         s.expire_all()
 
@@ -122,7 +142,7 @@ class ExpireTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session(autocommit=False)
+        s = fixture_session(autocommit=False)
         u = s.query(User).get(10)
 
         s.expire_all()
@@ -137,7 +157,7 @@ class ExpireTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session(autocommit=False)
+        s = fixture_session(autocommit=False)
         u = s.query(User).get(10)
         s.expire_all()
 
@@ -158,7 +178,7 @@ class ExpireTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session(autocommit=False)
+        s = fixture_session(autocommit=False)
         u = s.query(User).get(10)
         s.expire_all()
         s.execute(users.delete().where(User.id == 10))
@@ -186,19 +206,34 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"description": deferred(orders.c.description)},
         )
 
-        s = create_session()
+        s = fixture_session()
         o1 = s.query(Order).first()
         assert "description" not in o1.__dict__
         s.expire(o1)
+
+        # the deferred attribute is listed as expired (new in 1.4)
+        eq_(
+            inspect(o1).expired_attributes,
+            {"id", "isopen", "address_id", "user_id", "description"},
+        )
+
+        # unexpire by accessing isopen
         assert o1.isopen is not None
+
+        # all expired_attributes are cleared
+        eq_(inspect(o1).expired_attributes, set())
+
+        # but description wasn't loaded (new in 1.4)
         assert "description" not in o1.__dict__
+
+        # loads using deferred callable
         assert o1.description
 
     def test_deferred_notfound(self):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users, properties={"name": deferred(users.c.name)})
-        s = create_session(autocommit=False)
+        s = fixture_session(autocommit=False)
         u = s.query(User).get(10)
 
         assert "name" not in u.__dict__
@@ -230,7 +265,7 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        s = create_session(autoflush=True, autocommit=False)
+        s = fixture_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
         adlist = u.addresses
         eq_(
@@ -274,7 +309,7 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        s = create_session(autoflush=True, autocommit=False)
+        s = fixture_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
         assert_raises_message(
             sa_exc.InvalidRequestError,
@@ -284,18 +319,11 @@ class ExpireTest(_fixtures.FixtureTest):
             ["addresses"],
         )
 
-        # in contrast to a regular query with no columns
-        assert_raises_message(
-            sa_exc.InvalidRequestError,
-            "no columns with which to SELECT",
-            s.query().all,
-        )
-
     def test_refresh_cancels_expire(self):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(7)
         s.expire(u)
         s.refresh(u)
@@ -311,7 +339,7 @@ class ExpireTest(_fixtures.FixtureTest):
 
         mapper(User, users)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(7)
 
         sess.expire(u, attribute_names=["name"])
@@ -328,7 +356,7 @@ class ExpireTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(7)
 
         sess.expire(u, attribute_names=["name"])
@@ -341,7 +369,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # this was the opposite in 0.4, but the reasoning there seemed off.
         # expiring a pending instance makes no sense, so should raise
         mapper(User, users)
-        sess = create_session()
+        sess = fixture_session()
         u = User(id=15)
         sess.add(u)
         assert_raises(sa_exc.InvalidRequestError, sess.expire, u, ["name"])
@@ -354,7 +382,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # is actually part of a larger behavior when postfetch needs to
         # occur during a flush() on an instance that was just inserted
         mapper(User, users)
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(7)
 
         sess.expire(u, attribute_names=["name"])
@@ -370,7 +398,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # same as test_no_instance_key, but the PK columns
         # are absent.  ensure an error is raised.
         mapper(User, users)
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(7)
 
         sess.expire(u, attribute_names=["name", "id"])
@@ -387,7 +415,7 @@ class ExpireTest(_fixtures.FixtureTest):
         Order, orders = self.classes.Order, self.tables.orders
 
         mapper(Order, orders)
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         o = sess.query(Order).get(3)
         sess.expire(o)
 
@@ -439,11 +467,11 @@ class ExpireTest(_fixtures.FixtureTest):
 
         mapper(Order, orders)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         o = sess.query(Order).get(3)
         sess.expire(o)
 
-        orders.update().execute(description="order 3 modified")
+        sess.execute(orders.update(), dict(description="order 3 modified"))
         assert o.isopen == 1
         assert (
             attributes.instance_state(o).dict["description"]
@@ -473,7 +501,7 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        s = create_session()
+        s = fixture_session(autoflush=False)
         u = s.query(User).get(8)
         assert u.addresses[0].email_address == "ed@wood.com"
 
@@ -499,7 +527,7 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        s = create_session()
+        s = fixture_session(autoflush=False)
         u = s.query(User).get(8)
         assert u.addresses[0].email_address == "ed@wood.com"
 
@@ -537,7 +565,7 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"addresses": relationship(Address, cascade=cascade)},
         )
         mapper(Address, addresses)
-        s = create_session()
+        s = fixture_session(autoflush=False)
 
         u = s.query(User).get(8)
         a = Address(id=12, email_address="foobar")
@@ -570,7 +598,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(7)
 
         sess.expire(u)
@@ -605,7 +633,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(7)
 
         sess.expire(u)
@@ -616,9 +644,9 @@ class ExpireTest(_fixtures.FixtureTest):
             assert u.addresses[0].email_address == "jack@bean.com"
             assert u.name == "jack"
 
-        # two loads, since relationship() + scalar are
-        # separate right now on per-attribute load
-        self.assert_sql_count(testing.db, go, 2)
+        # one load, due to #1763 allows joinedload to
+        # take over
+        self.assert_sql_count(testing.db, go, 1)
         assert "name" in u.__dict__
         assert "addresses" in u.__dict__
 
@@ -653,7 +681,7 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(8)
         sess.expire(u, ["name", "addresses"])
         u.addresses
@@ -663,7 +691,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert "name" in u.__dict__
         assert len(u.addresses) == 2
 
-    def test_joinedload_props_dontload(self):
+    def test_mapper_joinedload_props_load(self):
         users, Address, addresses, User = (
             self.tables.users,
             self.classes.Address,
@@ -671,15 +699,7 @@ class ExpireTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        # relationships currently have to load separately from scalar instances
-        # the use case is: expire "addresses".  then access it.  lazy load
-        # fires off to load "addresses", but needs foreign key or primary key
-        # attributes in order to lazy load; hits those attributes, such as
-        # below it hits "u.id".  "u.id" triggers full unexpire operation,
-        # joinedloads addresses since lazy='joined'. this is all within lazy
-        # load which fires unconditionally; so an unnecessary joinedload (or
-        # lazyload) was issued.  would prefer not to complicate lazyloading to
-        # "figure out" that the operation should be aborted right now.
+        # changed in #1763, eager loaders are run when we unexpire
 
         mapper(
             User,
@@ -691,20 +711,76 @@ class ExpireTest(_fixtures.FixtureTest):
             },
         )
         mapper(Address, addresses)
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(8)
         sess.expire(u)
         u.id
-        assert "addresses" not in u.__dict__
+
+        assert "addresses" in u.__dict__
         u.addresses
         assert "addresses" in u.__dict__
 
+    def test_options_joinedload_props_load(self):
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+
+        # changed in #1763, eager loaders are run when we unexpire
+
+        mapper(
+            User,
+            users,
+            properties={"addresses": relationship(Address, backref="user")},
+        )
+        mapper(Address, addresses)
+        sess = fixture_session()
+        u = sess.query(User).options(joinedload(User.addresses)).get(8)
+        sess.expire(u)
+        u.id
+        assert "addresses" in u.__dict__
+        u.addresses
+        assert "addresses" in u.__dict__
+
+    def test_joinedload_props_load_two(self):
+        users, Address, addresses, User = (
+            self.tables.users,
+            self.classes.Address,
+            self.tables.addresses,
+            self.classes.User,
+        )
+        mapper(
+            User,
+            users,
+            properties={
+                "addresses": relationship(
+                    Address, backref="user", lazy="joined"
+                )
+            },
+        )
+        mapper(Address, addresses)
+        sess = fixture_session()
+        u = sess.query(User).get(8)
+        sess.expire(u)
+
+        # here, the lazy loader will encounter the attribute already
+        # loaded when it goes to get the PK, so the loader itself
+        # needs to no longer fire off.
+        def go():
+            u.addresses
+            assert "addresses" in u.__dict__
+            assert "id" in u.__dict__
+
+        self.assert_sql_count(testing.db, go, 1)
+
     def test_expire_synonym(self):
         User, users = self.classes.User, self.tables.users
 
         mapper(User, users, properties={"uname": sa.orm.synonym("name")})
 
-        sess = create_session()
+        sess = fixture_session()
         u = sess.query(User).get(7)
         assert "name" in u.__dict__
         assert u.uname == u.name
@@ -712,7 +788,7 @@ class ExpireTest(_fixtures.FixtureTest):
         sess.expire(u)
         assert "name" not in u.__dict__
 
-        users.update(users.c.id == 7).execute(name="jack2")
+        sess.execute(users.update(users.c.id == 7), dict(name="jack2"))
         assert u.name == "jack2"
         assert u.uname == "jack2"
         assert "name" in u.__dict__
@@ -728,7 +804,7 @@ class ExpireTest(_fixtures.FixtureTest):
 
         mapper(Order, orders)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         o = sess.query(Order).get(3)
 
         sess.expire(o, attribute_names=["description"])
@@ -736,7 +812,10 @@ class ExpireTest(_fixtures.FixtureTest):
         assert "description" not in o.__dict__
         assert attributes.instance_state(o).dict["isopen"] == 1
 
-        orders.update(orders.c.id == 3).execute(description="order 3 modified")
+        sess.execute(
+            orders.update(orders.c.id == 3),
+            dict(description="order 3 modified"),
+        )
 
         def go():
             assert o.description == "order 3 modified"
@@ -794,7 +873,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(8)
 
         sess.expire(u, ["name", "addresses"])
@@ -856,7 +935,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(8)
 
         sess.expire(u, ["name", "addresses"])
@@ -910,7 +989,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(8)
         assert "name" in u.__dict__
         u.addresses
@@ -937,31 +1016,32 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"description": sa.orm.deferred(orders.c.description)},
         )
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         o = sess.query(Order).get(3)
         sess.expire(o, ["description", "isopen"])
         assert "isopen" not in o.__dict__
         assert "description" not in o.__dict__
 
-        # test that expired attribute access refreshes
+        # test that expired attribute access does not refresh
         # the deferred
         def go():
             assert o.isopen == 1
             assert o.description == "order 3"
 
-        self.assert_sql_count(testing.db, go, 1)
+        # requires two statements
+        self.assert_sql_count(testing.db, go, 2)
 
         sess.expire(o, ["description", "isopen"])
         assert "isopen" not in o.__dict__
         assert "description" not in o.__dict__
-        # test that the deferred attribute triggers the full
+        # test that the deferred attribute does not trigger the full
         # reload
 
         def go():
             assert o.description == "order 3"
             assert o.isopen == 1
 
-        self.assert_sql_count(testing.db, go, 1)
+        self.assert_sql_count(testing.db, go, 2)
 
         sa.orm.clear_mappers()
 
@@ -1024,7 +1104,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u = sess.query(User).get(8)
         assert len(u.addresses) == 3
         sess.expire(u)
@@ -1056,7 +1136,7 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         userlist = sess.query(User).order_by(User.id).all()
         eq_(self.static.user_address_result, userlist)
         eq_(len(list(sess)), 9)
@@ -1070,13 +1150,15 @@ class ExpireTest(_fixtures.FixtureTest):
 
     def test_state_change_col_to_deferred(self):
         """Behavioral test to verify the current activity of loader
-        callables"""
+        callables
+
+        """
 
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
 
         # deferred attribute option, gets the LoadDeferredColumns
         # callable
@@ -1117,14 +1199,16 @@ class ExpireTest(_fixtures.FixtureTest):
         assert "name" not in attributes.instance_state(u1).callables
 
     def test_state_deferred_to_col(self):
-        """Behavioral test to verify the current activity of loader
-        callables"""
+        """Behavioral test to verify the current activity of
+        loader callables
+
+        """
 
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users, properties={"name": deferred(users.c.name)})
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u1 = sess.query(User).options(undefer(User.name)).first()
         assert "name" not in attributes.instance_state(u1).callables
 
@@ -1146,19 +1230,23 @@ class ExpireTest(_fixtures.FixtureTest):
         u1 = sess.query(User).options(undefer(User.name)).first()
         del u1.name
         sess.expire(u1)
-        assert "name" not in attributes.instance_state(u1).expired_attributes
+        assert "name" in attributes.instance_state(u1).expired_attributes
         assert "name" not in attributes.instance_state(u1).callables
 
         # single attribute expire, the attribute gets the callable
         sess.expunge_all()
         u1 = sess.query(User).options(undefer(User.name)).first()
         sess.expire(u1, ["name"])
+
+        # the expire cancels the undefer
         assert "name" in attributes.instance_state(u1).expired_attributes
         assert "name" not in attributes.instance_state(u1).callables
 
     def test_state_noload_to_lazy(self):
-        """Behavioral test to verify the current activity of loader
-        callables"""
+        """Behavioral test to verify the current activity of
+        loader callables
+
+        """
 
         users, Address, addresses, User = (
             self.tables.users,
@@ -1174,23 +1262,20 @@ class ExpireTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         u1 = sess.query(User).options(lazyload(User.addresses)).first()
         assert isinstance(
             attributes.instance_state(u1).callables["addresses"],
             strategies.LoadLazyAttribute,
         )
-        # expire, it stays
+        # expire, it goes away from callables as of 1.4 and is considered
+        # to be expired
         sess.expire(u1)
-        assert (
-            "addresses" not in attributes.instance_state(u1).expired_attributes
-        )
-        assert isinstance(
-            attributes.instance_state(u1).callables["addresses"],
-            strategies.LoadLazyAttribute,
-        )
 
-        # load over it.  callable goes away.
+        assert "addresses" in attributes.instance_state(u1).expired_attributes
+        assert "addresses" not in attributes.instance_state(u1).callables
+
+        # load it
         sess.query(User).first()
         assert (
             "addresses" not in attributes.instance_state(u1).expired_attributes
@@ -1223,7 +1308,7 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"description": deferred(orders.c.description)},
         )
 
-        s = Session()
+        s = fixture_session()
         item = Order(id=1)
 
         make_transient_to_detached(item)
@@ -1239,7 +1324,7 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"description": deferred(orders.c.description)},
         )
 
-        s = Session()
+        s = fixture_session()
 
         item = s.query(Order).first()
         s.expire(item)
@@ -1254,12 +1339,12 @@ class ExpireTest(_fixtures.FixtureTest):
             properties={"description": deferred(orders.c.description)},
         )
 
-        s = Session()
+        s = fixture_session()
 
         item = s.query(Order).first()
         s.expire(item, ["isopen", "description"])
         item.isopen
-        assert "description" in item.__dict__
+        assert "description" not in item.__dict__
 
 
 class PolymorphicExpireTest(fixtures.MappedTest):
@@ -1307,14 +1392,18 @@ class PolymorphicExpireTest(fixtures.MappedTest):
 
         connection.execute(
             people.insert(),
-            {"person_id": 1, "name": "person1", "type": "person"},
-            {"person_id": 2, "name": "engineer1", "type": "engineer"},
-            {"person_id": 3, "name": "engineer2", "type": "engineer"},
+            [
+                {"person_id": 1, "name": "person1", "type": "person"},
+                {"person_id": 2, "name": "engineer1", "type": "engineer"},
+                {"person_id": 3, "name": "engineer2", "type": "engineer"},
+            ],
         )
         connection.execute(
             engineers.insert(),
-            {"person_id": 2, "status": "new engineer"},
-            {"person_id": 3, "status": "old engineer"},
+            [
+                {"person_id": 2, "status": "new engineer"},
+                {"person_id": 3, "status": "old engineer"},
+            ],
         )
 
     @classmethod
@@ -1346,7 +1435,7 @@ class PolymorphicExpireTest(fixtures.MappedTest):
             self.classes.Engineer,
         )
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         [p1, e1, e2] = sess.query(Person).order_by(people.c.person_id).all()
 
         sess.expire(p1)
@@ -1388,7 +1477,7 @@ class PolymorphicExpireTest(fixtures.MappedTest):
     def test_no_instance_key(self):
         Engineer = self.classes.Engineer
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         e1 = sess.query(Engineer).get(2)
 
         sess.expire(e1, attribute_names=["name"])
@@ -1403,7 +1492,7 @@ class PolymorphicExpireTest(fixtures.MappedTest):
 
         # same as test_no_instance_key, but the PK columns
         # are absent.  ensure an error is raised.
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         e1 = sess.query(Engineer).get(2)
 
         sess.expire(e1, attribute_names=["name", "person_id"])
@@ -1435,7 +1524,7 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
         )
         mapper(Address, addresses)
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
         a1 = Address(email_address="a1")
         sess.add(a1)
         sess.flush()
@@ -1523,7 +1612,7 @@ class LifecycleTest(fixtures.MappedTest):
     def test_attr_not_inserted(self):
         Data = self.classes.Data
 
-        sess = create_session()
+        sess = fixture_session()
 
         d1 = Data()
         sess.add(d1)
@@ -1542,7 +1631,7 @@ class LifecycleTest(fixtures.MappedTest):
     def test_attr_not_inserted_expired(self):
         Data = self.classes.Data
 
-        sess = create_session()
+        sess = fixture_session(autoflush=False)
 
         d1 = Data()
         sess.add(d1)
@@ -1561,7 +1650,7 @@ class LifecycleTest(fixtures.MappedTest):
     def test_attr_not_inserted_fetched(self):
         Data = self.classes.DataFetched
 
-        sess = create_session()
+        sess = fixture_session()
 
         d1 = Data()
         sess.add(d1)
@@ -1578,15 +1667,12 @@ class LifecycleTest(fixtures.MappedTest):
     def test_cols_missing_in_load(self):
         Data = self.classes.Data
 
-        sess = create_session()
+        with Session(testing.db) as sess, sess.begin():
+            d1 = Data(data="d1")
+            sess.add(d1)
 
-        d1 = Data(data="d1")
-        sess.add(d1)
-        sess.flush()
-        sess.close()
-
-        sess = create_session()
-        d1 = sess.query(Data).from_statement(select([Data.id])).first()
+        sess = fixture_session()
+        d1 = sess.query(Data).from_statement(select(Data.id)).first()
 
         # cols not present in the row are implicitly expired
         def go():
@@ -1597,21 +1683,18 @@ class LifecycleTest(fixtures.MappedTest):
     def test_deferred_cols_missing_in_load_state_reset(self):
         Data = self.classes.DataDefer
 
-        sess = create_session()
+        with Session(testing.db) as sess, sess.begin():
+            d1 = Data(data="d1")
+            sess.add(d1)
 
-        d1 = Data(data="d1")
-        sess.add(d1)
-        sess.flush()
-        sess.close()
-
-        sess = create_session()
-        d1 = (
-            sess.query(Data)
-            .from_statement(select([Data.id]))
-            .options(undefer(Data.data))
-            .first()
-        )
-        d1.data = "d2"
+        with Session(testing.db) as sess:
+            d1 = (
+                sess.query(Data)
+                .from_statement(select(Data.id))
+                .options(undefer(Data.data))
+                .first()
+            )
+            d1.data = "d2"
 
         # the deferred loader has to clear out any state
         # on the col, including that 'd2' here
@@ -1641,7 +1724,7 @@ class RefreshTest(_fixtures.FixtureTest):
                 )
             },
         )
-        s = create_session()
+        s = fixture_session(autoflush=False)
         u = s.query(User).get(7)
         u.name = "foo"
         a = Address()
@@ -1676,7 +1759,7 @@ class RefreshTest(_fixtures.FixtureTest):
         users, User = self.tables.users, self.classes.User
 
         mapper(User, users)
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(7)
         s.expunge_all()
         assert_raises_message(
@@ -1685,11 +1768,29 @@ class RefreshTest(_fixtures.FixtureTest):
             lambda: s.refresh(u),
         )
 
+    def test_refresh_autoflush(self):
+        User, users = self.classes.User, self.tables.users
+        Address, addresses = self.classes.Address, self.tables.addresses
+
+        mapper(User, users)
+        mapper(Address, addresses, properties={"user": relationship(User)})
+
+        s = fixture_session()
+
+        a1 = s.query(Address).get(2)
+        u1 = s.query(User).get(7)
+        a1.user = u1
+
+        s.refresh(a1, ["user_id"])
+
+        # autoflushes
+        eq_(a1.user_id, 7)
+
     def test_refresh_expired(self):
         User, users = self.classes.User, self.tables.users
 
         mapper(User, users)
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(7)
         s.expire(u)
         assert "name" not in u.__dict__
@@ -1708,7 +1809,7 @@ class RefreshTest(_fixtures.FixtureTest):
             self.tables.users,
         )
 
-        s = create_session()
+        s = fixture_session()
         mapper(
             User,
             users,
@@ -1743,13 +1844,13 @@ class RefreshTest(_fixtures.FixtureTest):
             },
         )
 
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(8)
         assert len(u.addresses) == 3
         s.refresh(u)
         assert len(u.addresses) == 3
 
-        s = create_session()
+        s = fixture_session()
         u = s.query(User).get(8)
         assert len(u.addresses) == 3
         s.expire(u)
@@ -1773,7 +1874,7 @@ class RefreshTest(_fixtures.FixtureTest):
 
         mapper(Dingaling, dingalings)
 
-        s = create_session()
+        s = fixture_session()
         q = (
             s.query(User)
             .filter_by(name="fred")
@@ -1811,7 +1912,7 @@ class RefreshTest(_fixtures.FixtureTest):
             self.classes.User,
         )
 
-        s = create_session()
+        s = fixture_session()
         mapper(Address, addresses)
 
         mapper(
